@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
 
 from random import choice
 
-from django.utils.translation import ugettext_lazy as _
 from rest_framework.response import Response
 from rest_framework.generics import GenericAPIView, RetrieveUpdateAPIView
 
+from ..compat import gettext_lazy as _
 from ..docs import no_body, swagger_auto_schema
 from ..mixins import AppMixin
-from ..utils import get_app_model
-from .serializers import AppSerializer, AppKeySerializer
+from ..utils import get_app_model, get_app_serializer
+from .serializers import AppKeySerializer
 
 #pylint: disable=no-init
 
@@ -52,13 +52,13 @@ class GenerateKeyAPIView(AppMixin, GenericAPIView):
         Rotates the key used to encode the session information forwarded
         to the application entry point.
 
-        **Tags: rbac
+        **Tags: rbac, broker, appmodel
 
         **Examples
 
         .. code-block:: http
 
-            POST /api/proxy/key/ HTTP/1.1
+            POST /api/proxy/key HTTP/1.1
 
         responds
 
@@ -78,6 +78,8 @@ class GenerateKeyAPIView(AppMixin, GenericAPIView):
 
 class AppUpdateAPIView(AppMixin, RetrieveUpdateAPIView):
     """
+    Retrieves forward end-point
+
     Returns the URL endpoint to which requests passing the access rules
     are forwarded to, and the format in which the session information
     is encoded.
@@ -85,13 +87,13 @@ class AppUpdateAPIView(AppMixin, RetrieveUpdateAPIView):
     When running tests, you can retrieve the actual session information
     for a specific user through the `/proxy/sessions/{user}/` API call.
 
-    **Tags: rbac
+    **Tags: rbac, broker, appmodel
 
     **Examples
 
     .. code-block:: http
 
-        GET /api/proxy/ HTTP/1.1
+        GET /api/proxy HTTP/1.1
 
     responds
 
@@ -104,7 +106,7 @@ class AppUpdateAPIView(AppMixin, RetrieveUpdateAPIView):
         }
     """
     model = get_app_model()
-    serializer_class = AppSerializer
+    serializer_class = get_app_serializer()
 
     def get_object(self):
         return self.app
@@ -117,13 +119,13 @@ class AppUpdateAPIView(AppMixin, RetrieveUpdateAPIView):
         are forwarded to and/or the format in which the session information
         is encoded.
 
-        **Tags: rbac
+        **Tags: rbac, broker, appmodel
 
         **Examples
 
         .. code-block:: http
 
-            PUT /api/proxy/ HTTP/1.1
+            PUT /api/proxy HTTP/1.1
 
         .. code-block:: json
 
